@@ -70,3 +70,8 @@ list(map(lambda: 1, "xyz"))
 list(map(lambda x, y: x, [(1, 2), (3, 4)]))
 list(map(lambda: 1, "xyz"))
 list(map(lambda x, y: x, [(1, 2), (3, 4)]))
+
+# When inside t-string, then the fix should be surrounded by whitespace
+_ = t"{set(map(lambda x: x % 2 == 0, nums))}"
+_ = t"{dict(map(lambda v: (v, v**2), nums))}"
+
