@@ -3,20 +3,17 @@ use std::fmt::Display;
 use std::io::Write;
 use std::ops::Deref;
 
-use ruff_db::diagnostic::{
-    Annotation, Diagnostic, DiagnosticId, LintName, SecondaryCode, Severity, Span,
-};
 use rustc_hash::FxHashMap;
 
-pub use azure::AzureEmitter;
+use ruff_db::diagnostic::{
+    Annotation, Diagnostic, DiagnosticId, FileResolver, Input, LintName, SecondaryCode, Severity,
+    Span, UnifiedFile,
+};
+use ruff_db::files::File;
+
 pub use github::GithubEmitter;
 pub use gitlab::GitlabEmitter;
 pub use grouped::GroupedEmitter;
-pub use json::JsonEmitter;
-pub use json_lines::JsonLinesEmitter;
-pub use junit::JunitEmitter;
-pub use pylint::PylintEmitter;
-pub use rdjson::RdjsonEmitter;
 use ruff_notebook::NotebookIndex;
 use ruff_source_file::{LineColumn, SourceFile};
 use ruff_text_size::{Ranged, TextRange, TextSize};
@@ -26,24 +23,17 @@ pub use text::TextEmitter;
 use crate::Fix;
 use crate::registry::Rule;
 
-mod azure;
 mod diff;
 mod github;
 mod gitlab;
 mod grouped;
-mod json;
-mod json_lines;
-mod junit;
-mod pylint;
-mod rdjson;
 mod sarif;
 mod text;
 
 /// Creates a `Diagnostic` from a syntax error, with the format expected by Ruff.
 ///
 /// This is almost identical to `ruff_db::diagnostic::create_syntax_error_diagnostic`, except the
-/// `message` is stored as the primary diagnostic message instead of on the primary annotation, and
-/// `SyntaxError: ` is prepended to the message.
+/// `message` is stored as the primary diagnostic message instead of on the primary annotation.
 ///
 /// TODO(brent) These should be unified at some point, but we keep them separate for now to avoid a
 /// ton of snapshot changes while combining ruff's diagnostic type with `Diagnostic`.
@@ -52,11 +42,7 @@ pub fn create_syntax_error_diagnostic(
     message: impl std::fmt::Display,
     range: impl Ranged,
 ) -> Diagnostic {
-    let mut diag = Diagnostic::new(
-        DiagnosticId::InvalidSyntax,
-        Severity::Error,
-        format_args!("SyntaxError: {message}"),
-    );
+    let mut diag = Diagnostic::new(DiagnosticId::InvalidSyntax, Severity::Error, message);
     let span = span.into().with_range(range.range());
     diag.annotate(Annotation::primary(span));
     diag
@@ -83,6 +69,22 @@ where
         body,
     );
 
+    let span = Span::from(file).with_range(range);
+    let mut annotation = Annotation::primary(span);
+    // The `0..0` range is used to highlight file-level diagnostics.
+    //
+    // TODO(brent) We should instead set this flag on annotations for individual lint rules that
+    // actually need it, but we need to be able to cache the new diagnostic model first. See
+    // https://github.com/astral-sh/ruff/issues/19688.
+    if range == TextRange::default() {
+        annotation.set_file_level(true);
+    }
+    diagnostic.annotate(annotation);
+
+    if let Some(suggestion) = suggestion {
+        diagnostic.help(suggestion);
+    }
+
     if let Some(fix) = fix {
         diagnostic.set_fix(fix);
     }
@@ -95,18 +97,43 @@ where
         diagnostic.set_noqa_offset(noqa_offset);
     }
 
-    let span = Span::from(file).with_range(range);
-    let mut annotation = Annotation::primary(span);
-    if let Some(suggestion) = suggestion {
-        annotation = annotation.message(suggestion);
-    }
-    diagnostic.annotate(annotation);
-
     diagnostic.set_secondary_code(SecondaryCode::new(rule.noqa_code().to_string()));
 
     diagnostic
 }
 
+impl FileResolver for EmitterContext<'_> {
+    fn path(&self, _file: File) -> &str {
+        unimplemented!("Expected a Ruff file for rendering a Ruff diagnostic");
+    }
+
+    fn input(&self, _file: File) -> Input {
+        unimplemented!("Expected a Ruff file for rendering a Ruff diagnostic");
+    }
+
+    fn notebook_index(&self, file: &UnifiedFile) -> Option<NotebookIndex> {
+        match file {
+            UnifiedFile::Ty(_) => {
+                unimplemented!("Expected a Ruff file for rendering a Ruff diagnostic")
+            }
+            UnifiedFile::Ruff(file) => self.notebook_indexes.get(file.name()).cloned(),
+        }
+    }
+
+    fn is_notebook(&self, file: &UnifiedFile) -> bool {
+        match file {
+            UnifiedFile::Ty(_) => {
+                unimplemented!("Expected a Ruff file for rendering a Ruff diagnostic")
+            }
+            UnifiedFile::Ruff(file) => self.notebook_indexes.get(file.name()).is_some(),
+        }
+    }
+
+    fn current_directory(&self) -> &std::path::Path {
+        crate::fs::get_cwd()
+    }
+}
+
 struct MessageWithLocation<'a> {
     message: &'a Diagnostic,
     start_location: LineColumn,
@@ -122,7 +149,7 @@ impl Deref for MessageWithLocation<'_> {
 
 fn group_diagnostics_by_filename(
     diagnostics: &[Diagnostic],
-) -> BTreeMap<String, Vec<MessageWithLocation>> {
+) -> BTreeMap<String, Vec<MessageWithLocation<'_>>> {
     let mut grouped_messages = BTreeMap::default();
     for diagnostic in diagnostics {
         grouped_messages
