@@ -1,5 +1,7 @@
 use crate::Db;
-use crate::combine::Combine;
+use crate::glob::{
+    AbsolutePortableGlobPattern, PortableGlobError, PortableGlobKind, PortableGlobPattern,
+};
 use ruff_db::system::{System, SystemPath, SystemPathBuf};
 use ruff_macros::Combine;
 use ruff_text_size::{TextRange, TextSize};
@@ -7,10 +9,12 @@ use serde::{Deserialize, Deserializer};
 use std::cell::RefCell;
 use std::cmp::Ordering;
 use std::fmt;
+use std::fmt::Formatter;
 use std::hash::{Hash, Hasher};
 use std::ops::{Deref, DerefMut};
 use std::sync::Arc;
 use toml::Spanned;
+use ty_combine::Combine;
 
 #[derive(Clone, Debug)]
 pub enum ValueSource {
@@ -23,6 +27,9 @@ pub enum ValueSource {
     /// The value comes from a CLI argument, while it's left open if specified using a short argument,
     /// long argument (`--extra-paths`) or `--config key=value`.
     Cli,
+
+    /// The value comes from an LSP client configuration.
+    PythonVSCodeExtension,
 }
 
 impl ValueSource {
@@ -30,8 +37,13 @@ impl ValueSource {
         match self {
             ValueSource::File(path) => Some(&**path),
             ValueSource::Cli => None,
+            ValueSource::PythonVSCodeExtension => None,
         }
     }
+
+    pub const fn is_cli(&self) -> bool {
+        matches!(self, ValueSource::Cli)
+    }
 }
 
 thread_local! {
@@ -97,6 +109,14 @@ impl<T> RangedValue<T> {
         Self::with_range(value, ValueSource::Cli, TextRange::default())
     }
 
+    pub fn python_extension(value: T) -> Self {
+        Self::with_range(
+            value,
+            ValueSource::PythonVSCodeExtension,
+            TextRange::default(),
+        )
+    }
+
     pub fn with_range(value: T, source: ValueSource, range: TextRange) -> Self {
         Self {
             value,
@@ -319,11 +339,23 @@ impl RelativePathBuf {
         Self::new(path, ValueSource::Cli)
     }
 
+    pub fn python_extension(path: impl AsRef<SystemPath>) -> Self {
+        Self::new(path, ValueSource::PythonVSCodeExtension)
+    }
+
     /// Returns the relative path as specified by the user.
     pub fn path(&self) -> &SystemPath {
         &self.0
     }
 
+    pub fn source(&self) -> &ValueSource {
+        self.0.source()
+    }
+
+    pub fn range(&self) -> Option<TextRange> {
+        self.0.range()
+    }
+
     /// Returns the owned relative path.
     pub fn into_path_buf(self) -> SystemPathBuf {
         self.0.into_inner()
@@ -338,9 +370,71 @@ impl RelativePathBuf {
     pub fn absolute(&self, project_root: &SystemPath, system: &dyn System) -> SystemPathBuf {
         let relative_to = match &self.0.source {
             ValueSource::File(_) => project_root,
-            ValueSource::Cli => system.current_directory(),
+            ValueSource::Cli | ValueSource::PythonVSCodeExtension => system.current_directory(),
         };
 
         SystemPath::absolute(&self.0, relative_to)
     }
 }
+
+impl fmt::Display for RelativePathBuf {
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        self.0.fmt(f)
+    }
+}
+
+#[derive(
+    Debug,
+    Clone,
+    serde::Serialize,
+    serde::Deserialize,
+    PartialEq,
+    Eq,
+    PartialOrd,
+    Ord,
+    Hash,
+    Combine,
+)]
+#[serde(transparent)]
+#[cfg_attr(feature = "schemars", derive(schemars::JsonSchema))]
+pub struct RelativeGlobPattern(RangedValue<String>);
+
+impl RelativeGlobPattern {
+    pub fn new(pattern: impl AsRef<str>, source: ValueSource) -> Self {
+        Self(RangedValue::new(pattern.as_ref().to_string(), source))
+    }
+
+    pub fn cli(pattern: impl AsRef<str>) -> Self {
+        Self::new(pattern, ValueSource::Cli)
+    }
+
+    pub(crate) fn source(&self) -> &ValueSource {
+        self.0.source()
+    }
+
+    pub(crate) fn range(&self) -> Option<TextRange> {
+        self.0.range()
+    }
+
+    /// Resolves the absolute pattern for `self` based on its origin.
+    pub(crate) fn absolute(
+        &self,
+        project_root: &SystemPath,
+        system: &dyn System,
+        kind: PortableGlobKind,
+    ) -> Result<AbsolutePortableGlobPattern, PortableGlobError> {
+        let relative_to = match &self.0.source {
+            ValueSource::File(_) => project_root,
+            ValueSource::Cli | ValueSource::PythonVSCodeExtension => system.current_directory(),
+        };
+
+        let pattern = PortableGlobPattern::parse(&self.0, kind)?;
+        Ok(pattern.into_absolute(relative_to))
+    }
+}
+
+impl std::fmt::Display for RelativeGlobPattern {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        self.0.fmt(f)
+    }
+}
